import mapboxgl from 'mapbox-gl';
import { Matrix4, Group, PerspectiveCamera } from 'three';
export interface CameraSyncState {
    /**
     * @default Constant.FOV
     * @type {number}
     */
    fov?: number;
    /**
     * @default THREE.Matrix4().makeTranslation(Constant.WORLD_SIZE / 2, -Constant.WORLD_SIZE / 2, 0)
     * @type {Matrix4}
     */
    translateCenter?: Matrix4;
    /**
     * @default Constant.TILE_SIZE / Constant.WORLD_SIZE
     * @type {number}
     */
    worldSizeRatio?: number;
    cameraToCenterDistance?: number;
    offset?: {
        x: number;
        y: number;
    };
    cameraTranslateZ?: Matrix4;
    maxFurthestDistance?: number;
}
declare class CameraSync {
    map: mapboxgl.Map;
    camera: PerspectiveCamera;
    world: Group;
    active: boolean;
    state: CameraSyncState;
    cameraTranslateZ: number;
    constructor(map: mapboxgl.Map, camera: PerspectiveCamera, world?: Group);
    /**
     * Set up the camera
     *
     * @author salgum1114
     */
    setupCamera(): void;
    updateCamera(): void;
    /**
     * Calculate Camera Matrix
     *
     * @author salgum1114
     * @param {number} [pitch]
     * @param {number} [angle]
     * @param {Matrix4} [translateZ]
     * @returns
     */
    calcCameraMatrix(pitch?: number, angle?: number, translateZ?: Matrix4): Matrix4;
}
export default CameraSync;
