import { Matrix4, Vector3, Object3D, PerspectiveCamera } from 'three';
import Validator from './Validator';
import { ThreeboxObject } from './Threebox';
declare class Utils {
    Validator: Validator;
    exposedMethods: string[];
    /**
     *
     *
     * @author salgum1114
     * @param {number[]} uglymatrix
     */
    prettyPrintMatrix(uglymatrix: number[]): void;
    /**
     *
     *
     * @author salgum1114
     * @param {number} fovy
     * @param {number} aspect
     * @param {number} near
     * @param {number} far
     * @returns
     */
    makePerspectiveMatrix(fovy: number, aspect: number, near: number, far: number): Matrix4;
    /**
     * gimme radians
     *
     * @author salgum1114
     * @param {(number | [number, number, number] | { x: number; y: number; z: number })} deg
     * @returns
     */
    radify(deg: number | [number, number, number] | {
        x: number;
        y: number;
        z: number;
    }): number | number[];
    /**
     * gimme degrees
     *
     * @author salgum1114
     * @param {(number | { x: number; y: number; z: number })} rad
     * @returns
     */
    degreeify(rad: number | {
        x: number;
        y: number;
        z: number;
    }): number | number[];
    /**
     *
     *
     * @author salgum1114
     * @param {[number, number, number]} coords
     * @returns
     */
    projectToWorld(coords: [number, number, number]): Vector3;
    /**
     *
     *
     * @author salgum1114
     * @param {number} latitude
     * @returns
     */
    projectedUnitsPerMeter(latitude: number): number;
    /**
     *
     *
     * @author salgum1114
     * @param {[number, number, number]} centerLatLng
     * @param {Vector3[]} vertices
     * @returns
     */
    scaleVerticesToMeters(centerLatLng: [number, number, number], vertices: Vector3[]): Vector3[];
    /**
     * @deprecated
     *
     * @author salgum1114
     * @param {[number, number, number]} coords
     */
    projectToScreen(coords: [number, number, number]): void;
    /**
     * @deprecated
     *
     * @author salgum1114
     * @param {[number, number, number]} pixel
     */
    unprojectFromScreen(pixel: [number, number, number]): void;
    /**
     * world units to lnglat
     *
     * @author salgum1114
     * @param {{ x: number; y: number; z: number }} worldUnits
     * @returns
     */
    unprojectFromWorld(worldUnits: {
        x: number;
        y: number;
        z: number;
    }): number[];
    /**
     * @deprecated
     *
     * @author salgum1114
     * @param {Object3D} obj
     * @param {PerspectiveCamera} camera
     * @returns
     */
    toScreenPosition(obj: Object3D, camera: PerspectiveCamera): void;
    /**
     * get the center point of a feature
     *
     * @author salgum1114
     * @param {GeoJSON.Feature<any>} feature
     * @param {ThreeboxObject} [obj]
     * @param {number} [level]
     * @returns
     */
    getFeatureCenter(feature: GeoJSON.Feature<any>, obj?: ThreeboxObject, level?: number): number[];
    /**
     *
     *
     * @author salgum1114
     * @param {GeoJSON.Feature<any>} feature
     * @param {ThreeboxObject} obj
     * @param {number} [level=feature.properties.level || 0]
     * @returns
     */
    getObjectHeightOnFloor(feature: GeoJSON.Feature<any>, obj?: ThreeboxObject, level?: any): number;
    /**
     * @deprecated
     *
     * @author salgum1114
     * @param {*} obj
     */
    flipMaterialSides(obj: any): void;
    /**
     * to improve precision, normalize a series of vector3's to their collective center, and move the resultant mesh to that center
     *
     * @author salgum1114
     * @param {Vector3[]} vertices
     * @returns
     */
    normalizeVertices(vertices: Vector3[]): {
        vertices: Vector3[];
        position: Vector3;
    };
    /**
     * flatten an array of Vector3's into a shallow array of values in x-y-z order, for bufferGeometry
     *
     * @author salgum1114
     * @param {Vector3[]} vectors
     * @returns
     */
    flattenVectors(vectors: Vector3[]): [number?, number?, number?];
    lnglatsToWorld(coords: any[]): Vector3[];
    /**
     *
     *
     * @author salgum1114
     * @param {Record<string, any>} original
     * @param {Record<string, any>} addition
     */
    extend(original: Record<string, any>, addition: Record<string, any>): void;
    /**
     *
     *
     * @author salgum1114
     * @param {Record<string, any>} original
     * @returns
     */
    clone(original: Record<string, any>): Record<string, any>;
    /**
     *
     *
     * @author salgum1114
     * @param {number} n
     * @param {number} min
     * @param {number} max
     * @returns
     */
    clamp(n: number, min: number, max: number): number;
    /**
     * retrieve object parameters from an options object
     *
     * @author salgum1114
     * @param {(number | { x: number; y: number; z: number })} rotation
     * @param {[number, number, number]} currentRotation
     * @returns
     */
    rotation(rotation: number | {
        x: number;
        y: number;
        z: number;
    }, currentRotation: [number, number, number]): number | number[];
    /**
     * retrieve object parameters from an options object
     *
     * @author salgum1114
     * @param {(number | { x: number; y: number; z: number })} scale
     * @param {[number, number, number]} currentScale
     * @returns
     */
    scale(scale: number | {
        x: number;
        y: number;
        z: number;
    }, currentScale: [number, number, number]): number[];
    /**
     * retrieve object parameters from an options object
     *
     * @author salgum1114
     * @param {[number, number, number]} array
     * @param {[number, number, number]} current
     * @returns
     */
    applyDefault(array: [number, number, number], current: [number, number, number]): [number, number, number];
    /**
     *
     *
     * @author salgum1114
     * @param {Record<string, any>} userInputs
     * @param {Record<string, any>} defaults
     * @returns
     */
    validate(userInputs: Record<string, any>, defaults: Record<string, any>): Record<string, any>;
}
declare const _default: Utils;
export default _default;
