/// <reference types="mapbox-gl" />
import { LineBasicMaterial, BufferGeometry, Line, Group } from 'three';
import Threebox, { ThreeboxObject } from '../Threebox';
export declare type ObjectType = 'colors' | 'materials' | 'line' | 'sphere' | 'label' | 'tooltip' | 'tube' | 'extrusion' | 'loadModel' | 'object3D';
declare class ObjectHandler {
    threebox: Threebox;
    map: mapboxgl.Map;
    defaults: {
        [key in ObjectType]: Record<string, any>;
    };
    geometries: {
        line: string[];
        tube: string[];
        sphere: string[];
    };
    constructor(threebox: Threebox, map: mapboxgl.Map);
    line(obj: any): Line<BufferGeometry, LineBasicMaterial>;
    /**
     * @deprecated
     *
     * @author salgum1114
     * @param {*} option
     */
    extrusion(option: any): void;
    addMethods(obj: ThreeboxObject, isStatic?: boolean): ThreeboxObject;
    /**
     *
     *
     * @author salgum1114
     * @param {(any | any[])} obj
     * @param {*} option
     * @returns
     */
    makeGroup(obj: any | any[], option: any): Group;
    /**
     * add tooltip method
     *
     * @author salgum1114
     * @param {string} tooltipText
     * @param {boolean} [mapboxStyle=false]
     * @returns
     */
    drawTooltip(tooltipText: string, mapboxStyle?: boolean): HTMLSpanElement;
    /**
     * draw label method can be invoked separately
     *
     * @author salgum1114
     * @param {(HTMLElement | string)} element
     * @param {string} cssClass
     * @returns
     */
    drawLabelHTML(element: HTMLElement | string, cssClass: string): HTMLDivElement;
}
export default ObjectHandler;
